/* 
 * Copyright (c) 2009, Artem Shinkarov <artyom.shinkaroff@gmail.com>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* 
 * We do not want to have debug output, if you do, 
 * then you should define __DEBUG__ macro
 */
//#undef __DEBUG__        
/*
 * We do not want to test the library, so we do not
 * define __TEST_BITS__ macro.
 */
#undef __TEST_BITS__

#include "bits.h"

/* This comes from Linux include file <bits/byteswap.h> 
 * which is not in a standard library of BSD-like systems
 */

/* Swap bytes in 32 bit value.  */
#define __bswap_constant_32(x) \
     ((((x) & 0xff000000) >> 24) | (((x) & 0x00ff0000) >>  8) |		      \
      (((x) & 0x0000ff00) <<  8) | (((x) & 0x000000ff) << 24))

#ifdef __GNUC__
# if __GNUC__ >= 2
/* To swap the bytes in a word the i486 processors and up provide the
   `bswap' opcode.  On i386 we have to use three instructions.  */
#  if !defined __i486__ && !defined __pentium__ && !defined __pentiumpro__ \
      && !defined __pentium4__ && !defined __k8__ && !defined __athlon__ \
      && !defined __k6__
#   define __bswap_32(x)						      \
     (__extension__							      \
      ({ register unsigned int __v, __x = (x);				      \
	 if (__builtin_constant_p (__x))				      \
	   __v = __bswap_constant_32 (__x);				      \
	 else								      \
	   __asm__ ("rorw $8, %w0;"					      \
		    "rorl $16, %0;"					      \
		    "rorw $8, %w0"					      \
		    : "=r" (__v)					      \
		    : "0" (__x)						      \
		    : "cc");						      \
	 __v; }))
#  else
#   define __bswap_32(x) \
     (__extension__							      \
      ({ register unsigned int __v, __x = (x);				      \
	 if (__builtin_constant_p (__x))				      \
	   __v = __bswap_constant_32 (__x);				      \
	 else								      \
	   __asm__ ("bswap %0" : "=r" (__v) : "0" (__x));		      \
	 __v; }))
#  endif
# else
#  define __bswap_32(x) \
     (__extension__							      \
      ({ register unsigned int __x = (x); __bswap_constant_32 (__x); }))
# endif
#else
static __inline unsigned int
__bswap_32 (unsigned int __bsx)
{
  return __bswap_constant_32 (__bsx);
}
#endif



const char *bits_error_table[3] =
{
    "Successfully read",
    "Attempt to read before or after bit array",
    "Size error, you can read only 1..32 bits per time."
};

/*
 * Initializing data structure. Allocating memory.
 */

/*@null@*/ 
struct bits * 
init_bits() {
    /*@null@*/
    struct bits *ba=NULL;

    if ( NULL == ( ba = (struct bits *)malloc(sizeof(struct bits)) ) ) {
        warnx("memory exhausted");
        return NULL;
    }
    else {
        ba->buf = ba->bufPtr = NULL;
        ba->bitPtr = ba->size = 0;
        return ba;
    }
}

/*
 * Deallocating memory.
 */
void
free_bits(struct bits * ba) {
    if (ba->buf != NULL)
        free(ba->buf);

    ba->bufPtr = NULL;
    ba->bitPtr = ba->size = 0;
}



/*
 * Reading n bits from `bits' structure, and returning them 
 * as uint32. n must be [1..32].
 *
 * |10110000|11110100|11...
 * ^
 * bufPtr
 *
 * bitPtr = 3, we have read 3 bytes already (101)
 * If we are requesting 10 bytes, we will get an uint32_t:
 * 0..01000011110
 *
 * Function moves after successful reading bufPtr and bitPtr
 *
 */
inline uint32_t __attribute__ ((always_inline))
_get_bits(struct bits *ba,     uint8_t  n, /*@out@*/ int *error,
         uint8_t     **bufptr, uint8_t *bitptr) {
    
    uint32_t         res;
    uint8_t         *bufPtr;
    uint8_t          bitPtr;

    res  = 0;
    *error = BITS_OK;
    
    if (n > 32) {
        *error = BITS_SIZEERROR;
        return 0;
    }
    
    /* 
     * Saving the position, for returning in the same 
     * state in case some error would occur. 
     */
    bitPtr = bitptr == NULL? ba->bitPtr: *bitptr;
    bufPtr = bufptr == NULL? ba->bufPtr: *bufptr;

    if ((size_t)bufPtr + (bitPtr + n) / 8 - (size_t)ba->buf >= ba->size) {
        *error = BITS_CANNOTREAD;
        return 0;
    }

#if 0    
    while ( n > 0 ) {
        /*if ((size_t)bufPtr - (size_t)ba->buf >= ba->size) {
            *error = BITS_CANNOTREAD;
            return 0;
        }*/
        
        if ( n <= 8 - bitPtr  ) {
            res <<= n;
            res += (uint32_t)(uint8_t)
                   ((uint8_t)(*bufPtr << bitPtr) >> (8 - n));
            bufPtr = (n == 8 - bitPtr)? ++bufPtr : bufPtr;
            bitPtr = (bitPtr + n) % 8;
            n = 0;
        } else {
            res <<= (8 - bitPtr);
            res += (uint32_t)(uint8_t)
                   ((uint8_t)(*bufPtr << bitPtr) >> bitPtr);
            bufPtr++;
            n -= (8 - bitPtr);
            bitPtr = 0;
        }
    }
#endif
    

#if 0
    uint64_t tmp, tmp1=0;

    tmp = (*(uint64_t *)bufPtr);
    /*tmp = (tmp>>56) | 
          ((tmp<<40) & 0x00FF000000000000) |
          ((tmp<<24) & 0x0000FF0000000000) |
          ((tmp<<8)  & 0x000000FF00000000) |
          ((tmp>>8)  & 0x00000000FF000000) |
          ((tmp>>24) & 0x0000000000FF0000) |
          ((tmp>>40) & 0x000000000000FF00) |
          (tmp<<56);*/

    tmp = (uint64_t)(tmp << 40) |
          (uint64_t)((tmp << 24) & (uint64_t)0x000000ff00000000L) |
          (uint64_t)((tmp << 8 ) & (uint64_t)0x00000000ff000000L) |
          (uint64_t)((tmp >> 8 ) & (uint64_t)0x0000000000ff0000L) |
          (uint64_t)((tmp >> 24) & (uint64_t)0x000000000000ff00L) |
          (uint64_t)((tmp >> 40) & (uint64_t)0x00000000000000ffL)
          ;


    /*tmp1 |= (tmp & 0xff) << 40 | ((tmp >> 8) & 0xff) << 32 | 
           ((tmp >> 16) & 0xff) << 24 | ((tmp >> 24) & 0xff) << 16 |
           ((tmp >> 32) &0xff) << 8 | (tmp >> 40) & 0xff;*/

    res = ((uint32_t)(tmp >> (48 - n - bitPtr))) & ((1 << n)-1);
    

    bufPtr += (bitPtr + n) / 8;
    bitPtr = (bitPtr + n) % 8;
#endif

#if 1
//#include <byteswap.h>
    uint32_t tmp;
    uint8_t tmp1;
    if  (bitPtr + n <= 32) {
        tmp = *(uint32_t *)bufPtr;
        tmp = __bswap_32(tmp);
        //printf("#1 tmp  = %#x, bitPtr = %i, n = %i\n", tmp, bitPtr, n);
        res = (tmp >> (32 - n - bitPtr)) & ((1 << n) -1);
        //printf("   res  = %#x\n", res);
    }
    else {
        tmp = *(uint32_t *)bufPtr;
        tmp1 = *((uint8_t *)(((uint32_t *)bufPtr) + 1));
        tmp = __bswap_32(tmp);
        //printf("#2 tmp  = %#x, bitPtr = %i, n = %i\n", tmp, bitPtr, n);
        //printf("   tmp1 = %#x\n", tmp1);

        res = (((tmp << bitPtr) | (tmp1 >> (8 - bitPtr))) >> (32 - n)) 
              & ((1 << n) -1);
        
        //printf("   res  = %#x\n", res);
    }
    bufPtr += (bitPtr + n) / 8;
    bitPtr = (bitPtr + n) % 8;
   
#endif

    if (bufptr != NULL && bitptr != NULL) {
        *bufptr = bufPtr;
        *bitptr = bitPtr;
    }
    else {
        ba->bufPtr = bufPtr;
        ba->bitPtr = bitPtr;
    }
    //dprintf("finish\n");
    return res;
}


/* Interface for reading from current array location */
inline uint32_t
get_bits (struct bits *ba, uint8_t n, /*@out@*/ int *error) {
    return _get_bits(ba, n, error, NULL, NULL);
}



/*
 * Write n bits (n <= 8) into specified location of the 
 * array. All the rest bytes are not changed.
 *
 * Writing is from left to right.
 * For example:
 * ba->buf = |xxxxxxxx|xxxxxxxx|
 * -- write 3 bits (101)
 * ba->buf = |101xxxxx|xxxxxxxx|
 * -- write 7 bits (1011011)
 * ba->buf = |10110110|11xxxxxx|
 * And so on
 *
 * If when writing we need some more memory, it
 * is being allocated automativally, without any
 * possible troubles for user.
 *
 * Function sets an error variable during the execution
 * showing the success.
 */
void
_write_ubits(
    struct bits  *ba,     uint8_t data, uint8_t n, int *error,
    uint8_t     **bufptr, uint8_t *bitptr){
    
    uint8_t     *bufPtr;
    uint8_t      bitPtr;
    uint8_t      bitsWritten;

    if (n > 8) {
        *error  = BITS_SIZEERROR;
        return;
    }

    *error       = BITS_OK;
     bitsWritten = 0;

    /* 
     * The location in array is external or is
     * taken from the array
     */
    bufPtr = bufptr == NULL ? ba->bufPtr : *bufptr;
    bitPtr = bitptr == NULL ? ba->bitPtr : *bitptr;
    
    if ((8 - bitPtr) >= n) {
        /*
         * This is a situation like:
         * bufPtr = |110xxxxx|, and we need to put
         *              ^
         *              bitPtr
         *
         * n bits, n <= 5
         *
         * We are changing only the n bits, and allocating
         * additional byte of memory for ba->buf if we 
         * used the last available bit
         */
        
         *bufPtr = (  
                  *bufPtr &
                  ~((uint8_t)( ((uint8_t)(1 << n) -1) << (8 - bitPtr - n) ))
               )
               | 
               (( data << (8 - bitPtr - n) ) &
                ((uint8_t)( ((uint8_t)(1 << n) -1) << (8 - bitPtr - n) )))
               ;

        bitPtr = (bitPtr + n) % 8;
        
        if (bitPtr == 0) {
            bufPtr++;
            if ((size_t)bufPtr - (size_t)ba->buf == ba->size) {
                ba->size ++;
                ba->buf = (uint8_t *)
                      realloc(ba->buf, ba->size * sizeof(uint8_t));
                bufPtr = ba->buf + (ba->size - 1);
                /* Setting original bufPtr to valid address after realloc */
                ba->bufPtr = bufPtr -1;

            }
        }
    }
    else {
        /*
         * Here we have situation like:
         * bufPtr = |01011xxx|, and we need to write n bits
         * n > 3.
         *
         * In this case we decompose our writing in two stages
         * first we fill the available byte of bufPtr, then
         * we switch bufPtr to the next one and filling it
         * with the rest of bits.
         *
         * Memory is allocated if needed.
         */

        *bufPtr = (
                  *bufPtr &
                  ~((uint8_t)( (uint8_t)(1 << (8 - bitPtr)) - 1 ))
               )
               |
               (( data >> (n - 8 + bitPtr) ) &
                ((uint8_t)( (uint8_t)(1 << (8 - bitPtr)) - 1 )))
               ;

        
        bufPtr++;
        bitPtr = n - 8 + bitPtr;
        if ((size_t)bufPtr - (size_t)ba->buf == ba->size) {
            ba->size ++;
            ba->buf = (uint8_t *)
                  realloc(ba->buf, ba->size * sizeof(uint8_t));
            bufPtr = ba->buf + (ba->size - 1);
            /* Setting original bufPtr to valid address after realloc */
            ba->bufPtr = bufPtr - 1;

        }

        *bufPtr = (
                    *bufPtr &
                    ~(uint8_t)( ((uint8_t)(1 << bitPtr) - 1) << (8 - bitPtr) )
               )
               |
               (((uint8_t) data << (8 - bitPtr) ) &
                (uint8_t)( ((uint8_t)(1 << bitPtr) - 1) << (8 - bitPtr) ))
               ;
    }

    /* 
     * If the location was internal, then we move ba->bufPtr
     * and ba->bitPtr to the position which is after the last
     * written bit
     */
    if (bufptr != NULL && bitptr != NULL) {
        *bufptr = bufPtr;
        *bitptr = bitPtr;
    }
    else {
        ba->bufPtr = bufPtr;
        ba->bitPtr = bitPtr;
    }
}

/* Interface function for the internal location */
void write_ubyte (struct bits *ba, uint8_t data, int *error) {
    _write_ubits(ba, data, 8, error, NULL, NULL);
}

/* Move current array pointer */
void move_pointer(struct bits *ba, int bit_count, int*error) {
    int byte = (int)(ba->bitPtr + bit_count);
    bit_count = byte % 8;
    byte /= 8;
    //dprintf("%s, byte=%i, bitPtr=%i\n", __func__, byte, bit_count);
    if (bit_count < 0) {
        byte -= 1;
        bit_count += 8;
    }
    if ((ba->bufPtr + byte) < ba->buf ||
        (ba->bufPtr + byte - ba->buf) > (int)ba->size) {
        *error = BITS_SIZEERROR;
        return;
    }
    else {
        //dprintf("%s, byte=%i, bitPtr=%i\n", __func__, byte, bit_count);
        ba->bufPtr += byte;
        ba->bitPtr = (uint8_t)bit_count;
    }
}

#ifdef __TEST_BITS__
int main(int argc, char *argv[]) {
    
    struct bits     *ba;
    int              error;
    uint32_t         b1, b2, b3;

    printf("Allocating bits array.\n");
    ba = init_bits();
    ba->bufPtr = ba->buf = (uint8_t *)malloc(1 * sizeof(uint8_t));
    ba->size = 1;

    printf("Writing 3 bits [0x05], 2 bits [0x03], 8 bits, [0x2a]\n");
    
    _write_ubits(ba, '\x05', 3, &error, NULL, NULL);
    if (error != BITS_OK) 
        printf("error ocured, when writing [0x05], error code is %i\n", error);
    
    _write_ubits(ba, '\x03', 2, &error, NULL, NULL);
    if (error != BITS_OK) 
        printf("error ocured, when writing [0x03], error code is %i\n", error);
    
    write_ubyte(ba, '\x2a', &error);
    if (error != BITS_OK) 
        printf("error ocured, when writing [0x2a], error code is %i\n", error);
    
    printf("buffer is: [%i][%i][%i]\n", ba->buf[0], ba->buf[1], ba->buf[2]);
    /* Setting array's inner pointers to zero */
    ba->bufPtr = ba->buf;
    ba->bitPtr = 0;

    /* Reading bits we wrote */
    printf("Reading sequentialy 3 bits, 2 bits, and 8 bits\n");
    
    b1 = get_bits(ba, 3, &error);
    if (error != BITS_OK)
            printf("Error ocured, reading 3 bits\n");
    
    b2 = get_bits(ba, 2, &error);
    if (error != BITS_OK)
            printf("Error ocured, reading 2 bits\n");

    b3 = get_bits(ba, 8, &error);
    if (error != BITS_OK)
            printf("Error ocured, reading 8 bits\n");

    printf("get(3) = %u, get(2) = %u, get(8) = %u\n", b1, b2, b3);

    move_pointer(ba, -9, &error);
    b1 = get_bits(ba, 3, &error);
    b2 = get_bits(ba, 3, &error);

    printf("move(-9), get(3) = %u\n", b1);
    printf("one more  get(3) = %u\n", b2);
    free_bits(ba);

    return (EXIT_SUCCESS);
}
#endif
