
#include <sys/time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <err.h>
#include "bits.c"


struct timeval *
SACGetTimeVal() {
    struct timeval *t;
    t = (struct timeval *)malloc(sizeof(struct timeval));
    gettimeofday(t, (struct timezone *)NULL);
    return t;
}

void
SACPrintTimeDiff(struct timeval *before, struct timeval *after) {
    struct timeval *t;
    t = (struct timeval *)malloc(sizeof(struct timeval));
    timersub(after, before, t);
    fprintf(stdout,"SECONDS: %9ld.%03ld\n",
		   t->tv_sec, t->tv_usec/10000);
    free(t);
}

void
SACFreeTime(struct timeval *t) {
    if (t) free(t);
}


int
SACStderrWrite(char *array, int len) {
    return write(2, array, len);
}

int
SACStdoutWrite(char *array, int len) {
    return write(1, array, len);
}

struct bits *
SACReadStdinInput() {
    struct bits     *ba;
    char            *in;
    struct stat      st;
    int              fd, rc;
    size_t           bufsz, cursz, curpos;
    ssize_t          ssz;
    unsigned char    *buf, *bufp;

    in = "<stdin>";
    fd = fcntl(STDIN_FILENO, F_DUPFD, 0);

    if (fstat(fd, &st) >= 0)
        bufsz = (size_t)st.st_blksize;
    else
        warnx("%s", in);

    if (NULL == (buf = (unsigned char *)malloc(bufsz))) {
        warn("malloc");
        if (-1 == close(fd))
            warn("%s", in);
        exit(EXIT_FAILURE);
    }

    curpos = 0;
    cursz = bufsz;

    while ((ssz = read(fd, buf + curpos, bufsz)) > 0) {
        curpos += ssz;
        cursz = curpos + bufsz;
        if (NULL == (bufp = (unsigned char *)realloc(buf, cursz))) {
                warnx("memory exhausted");
                break;
        }
        buf = bufp;
    }

    if (-1 == ssz) {
            warn("%s", in);
            exit(EXIT_FAILURE);
    } else if (ssz || 0 == curpos)
            exit(EXIT_FAILURE);
    
    buf[curpos] = 0;

    ba = init_bits();
    ba->buf = ba->bufPtr = (unsigned char *) buf;
    ba->size = curpos;

    return ba;
}

int 
SACGetBits(struct bits *ba, int num) {
    int error;
    uint32_t res;
    res = get_bits(ba, num, &error);
    if (error != BITS_OK)                        
        errx(-1, "BITS Library ERROR:%s:%4u, %s\n",
                __FILE__, __LINE__,
                bits_error_table[error]);
    return (int)res;
}

int
SACBitsStrCompare(struct bits *ba, const char *str) {
    int len, i, r, res = 0, error;

    len = strlen(str);
    for (i = 0; i < len; i++) {
        r = SACGetBits(ba, 8);
        if ((int)(unsigned char)str[i] > r) {
            res = -1;
            break;
        }
        if ((int)(unsigned char)str[i] < r) {
            res = 1;
            break;
        }
    }
    if (res != 0)
        move_pointer(ba, -((i+1)*8), &error);
    return res;
}


void
SACMoveBitsPointer(struct bits *ba, int num) {
    int error;
    move_pointer(ba, num, &error);
    /*if (error != BITS_OK)                        
        errx(-1, "BITS Library ERROR:%s:%4u, %s\n",
                __FILE__, __LINE__,
                bits_error_table[error]);*/

}

void
SACFreeBits(struct bits *ba) {
    if (ba) {
        free_bits(ba);
        free(ba);
    }
}

int 
SACshl(int a, int b) {
    return a << b;
}

int 
SACshr(int a, int b) {
    return a >> b;
}

int 
SACand(int a, int b) {
    return a & b;
}
