#include"raytracing.h"

#define MAX_INPUT_LINE 512

extern void abvh(struct Scene *scene);

static int Skip( char *line )
{
	char c;

    for( c = *line; c != '\0' && c != '\n' && c != '#'; c = *++line )
        if( c != ' ' && c != '\t' ) return 0;

    return 1;
}

void material_copy( struct Material *det, struct Material *src )
{
	det->ambient = src->ambient;
	det->diffuse = src->diffuse;
	det->emission = src->emission;
	det->Phong_exp = src->Phong_exp;
	det->ref_index = src->ref_index;
	det->reflectivity = src->reflectivity;
	det->specular = src->specular;
	det->translucency = src->translucency;
}

void buildScene( char *DefaultScene, struct Camera **cam_p, struct Scene **scene_p )
{
  FILE *fp;
	int line_num = 0;
    char file_name[80], input_line[MAX_INPUT_LINE], word[80];
	struct Material material;
	int num_lights = 0, num_objects = 0;
	struct Color White, Black, defaultEnv;
        struct Camera *cam;
        struct Scene *scene;


        cam = (struct Camera*)malloc( sizeof( struct Camera));
        *cam_p = cam;
        scene = (struct Scene *)malloc( sizeof( struct Scene));
        *scene_p = scene;
        
	// Attempt to open the input file.
	strcpy( file_name, DefaultScene );
	printf( "Reading %s... \n", file_name );
        
	//if( !freopen( file_name, "r", stdin ) ) return 0;
        fp = fopen (file_name,"r");
        if(fp == NULL) {
          fprintf( stderr, "Cannot open file %s\n", DefaultScene);
          exit(1);
        } 

	// Set some defaults.
	scene->num_lights = 0;
	scene->num_objects = 0;
	colorSet( &defaultEnv, 0.15, 0.25, 0.35 );
	scene->env = defaultEnv;

	colorSet( &White, 1.0, 1.0, 1.0 );
	colorSet( &Black, 0.0, 0.0, 0.0 );
    material.diffuse      = White;
    material.emission     = Black;
    material.specular     = White;
    material.ambient      = Black;
    material.reflectivity = Black;
    material.translucency = Black;
    material.ref_index    = 0.0;
    material.Phong_exp    = 0.0;

	cam->x_res = 400;
    cam->y_res = 400;
	cam->x_win.min = -1.0; cam->x_win.max = 1.0;
    cam->y_win.min = -1.0; cam->y_win.max = 1.0;

    // Process lines until the end of file is reached.
    // Print a warning for all lines that are unrecognizable.

	//while( fgets( input_line, MAX_INPUT_LINE, stdin ) )
        while( fgets( input_line, MAX_INPUT_LINE, fp ) )  
	{
        line_num++;
        if( Skip( input_line ) ) continue;
        //printf("line_num = %d\n", line_num);
		word[0] = '\0';
		sscanf( input_line, "%s", word );

		if( !strcmp( word, "eye" ) )
			sscanf( input_line, "%s ( %lf, %lf, %lf )", word, &cam->eye.x, &cam->eye.y, &cam->eye.z );
		else if( !strcmp( word, "lookat" ) )
			sscanf( input_line, "%s ( %lf, %lf, %lf )", word, &cam->lookat.x, &cam->lookat.y, &cam->lookat.z );
		else if( !strcmp( word, "up" ) )
			sscanf( input_line, "%s ( %lf, %lf, %lf )", word, &cam->up.x, &cam->up.y, &cam->up.z );
		else if( !strcmp( word, "vpdist" ) )
			sscanf( input_line, "%s %lf", word, &cam->vpdist );
		else if( !strcmp( word, "x_win" ) )
			sscanf( input_line, "%s ( %lf, %lf )", word, &cam->x_win.min, &cam->x_win.max );
		else if( !strcmp( word, "y_win" ) )
			sscanf( input_line, "%s ( %lf, %lf )", word, &cam->y_win.min, &cam->y_win.max );
		else if( !strcmp( word, "x_res" ) )
			sscanf( input_line, "%s %d", word, &cam->x_res );
		else if( !strcmp( word, "y_res" ) )
			sscanf( input_line, "%s %d", word, &cam->y_res );

		else if( !strcmp( word, "envmap" ) )
			sscanf( input_line, "%s [ %lf, %lf, %lf ]", word, &scene->env.red, &scene->env.green, &scene->env.blue );
		else if( !strcmp( word, "diffuse" ) )
			sscanf( input_line, "%s [ %lf, %lf, %lf ]", word, &material.diffuse.red, &material.diffuse.green, &material.diffuse.blue);
		else if( !strcmp( word, "emission" ) )
			sscanf( input_line, "%s [ %lf, %lf, %lf ]", word, &material.emission.red, &material.emission.green, &material.emission.blue);
		else if( !strcmp( word, "specular" ) )
			sscanf( input_line, "%s [ %lf, %lf, %lf ]", word, &material.specular.red, &material.specular.green, &material.specular.blue);
		else if( !strcmp( word, "ambient" ) )
			sscanf( input_line, "%s [ %lf, %lf, %lf ]", word, &material.ambient.red, &material.ambient.green, &material.ambient.blue);
		else if( !strcmp( word, "reflectivity" ) )
			sscanf( input_line, "%s [ %lf, %lf, %lf ]", word, &material.reflectivity.red, &material.reflectivity.green, &material.reflectivity.blue);
		else if( !strcmp( word, "translucency" ) )
			sscanf( input_line, "%s [ %lf, %lf, %lf ]", word, &material.translucency.red, &material.translucency.green, &material.translucency.blue);
		else if( !strcmp( word, "ref_index" ) )
			sscanf( input_line, "%s %lf", word, &material.ref_index);
		else if( !strcmp( word, "Phong_exp" ) )
			sscanf( input_line, "%s %lf", word, &material.Phong_exp);

		else if( !strcmp( word, "point" ) )
		{
			strcpy( scene->object[num_objects].name, word );
			sscanf( input_line, "%s ( %lf, %lf, %lf )", word, &scene->object[num_objects].position.x, &scene->object[num_objects].position.y, &scene->object[num_objects].position.z);
			material_copy( &scene->object[num_objects].material, &material );
			if( material.emission.blue > 0 || material.emission.green > 0 || material.emission.red > 0 )
				scene->light[ num_lights++ ] = &scene->object[ num_objects ];
			num_objects++;
		}
		else if( !strcmp( word, "block" ) )
		{
			strcpy( scene->object[num_objects].name, word );
			sscanf( input_line, "%s ( %lf, %lf, %lf ) ( %lf, %lf, %lf )", word, 
				&scene->object[num_objects].Min.x, &scene->object[num_objects].Min.y, &scene->object[num_objects].Min.z,
				&scene->object[num_objects].Max.x, &scene->object[num_objects].Max.y, &scene->object[num_objects].Max.z
				);
			material_copy( &scene->object[num_objects].material, &material );
			if( material.emission.blue > 0 || material.emission.green > 0 || material.emission.red > 0 )
				scene->light[ num_lights++ ] = &scene->object[ num_objects ];
			num_objects++;

		}
		else if( !strcmp( word, "sphere" ) )
		{
			strcpy( scene->object[num_objects].name, word );
			sscanf( input_line, "%s ( %lf, %lf, %lf ) %lf ", word, 
				&scene->object[num_objects].center.x, &scene->object[num_objects].center.y, &scene->object[num_objects].center.z,
				&scene->object[num_objects].radius
				);
			material_copy( &scene->object[num_objects].material, &material );
			if( material.emission.blue > 0 || material.emission.green > 0 || material.emission.red > 0 )
				scene->light[ num_lights++ ] = &scene->object[ num_objects ];
			num_objects++;

		}
		else if( !strcmp( word, "triangle" ) )
		{
			strcpy( scene->object[num_objects].name, word );
			sscanf( input_line, "%s ( %lf, %lf, %lf ) ( %lf, %lf, %lf ) ( %lf, %lf, %lf )  ", word, 
				&scene->object[num_objects].A.x, &scene->object[num_objects].A.y, &scene->object[num_objects].A.z,
				&scene->object[num_objects].B.x, &scene->object[num_objects].B.y, &scene->object[num_objects].B.z,
				&scene->object[num_objects].C.x, &scene->object[num_objects].C.y, &scene->object[num_objects].C.z
				);

			material_copy( &scene->object[num_objects].material, &material );
			if( material.emission.blue > 0 || material.emission.green > 0 || material.emission.red > 0 )
				scene->light[ num_lights++ ] = &scene->object[ num_objects ];
			num_objects++;
		}


		//printf("%s\n", input_line);
	}
        fclose(fp);
        
	scene->num_objects = num_objects;
	scene->num_lights = num_lights;
	printf( "done.\n" );

	// Adding each of the objects to a the bounding volume hierarchy.
	abvh(scene);
	
}
