#include "raytracing.h"

bool sphere_intersect( struct Ray *ray, struct HitInfo *hitinfo, struct Object *obj )
{
	struct Vec3 A, R;
	double b, discr, radical, s;

    A = vec3Sub( ray->origin, obj->center );
    R = ray->direction;
    b = 2.0 * vec3InnerProduct( A, R );
	discr = b * b - 4.0 * ( vec3InnerProduct(A, A) - obj->radius * obj->radius );  // The discriminant.

    // If the discriminant if negative, the quadratic equation had negative
    // roots, and the ray misses the sphere.

    if( discr < 0.0 ) return 0;

    radical = sqrt( discr );

    // First try the smaller of the two roots.  If this is positive, it is the
    // closest intersection.

    s = 0.5 * ( -b - radical );
    if( s > 0.0 )
    {
        // If the closest intersection is too far away, report a miss.
		if( s > hitinfo->distance ) return 0;
    }
    else
    {
        // Now try the other root, since the smallest root puts the
        // point of intersection "behind"us.  If the larger root is now
        // positive, it means we are inside the sphere.
        s = 0.5 * ( -b + radical );
        if( s <= 0 ) return 0;
        if( s > hitinfo->distance ) return 0;
    }

	// We have an actual hit.  Fill in all the geometric information so
    // that the shader can shade this point.

    hitinfo->distance = s;
    hitinfo->point    = vec3Add( ray->origin, vec3Mul( s, R ) );
	hitinfo->normal   = Unit( vec3Sub(hitinfo->point, obj->center ) );
    hitinfo->object   = obj;
    return 1;
}

struct Interval sphere_GetSlab( struct Object *obj, struct Vec3 *v )
{
    double len = Length(*v);
	double dot = vec3InnerProduct(*v, obj->center) / len; //dot = ( v * center ) / len;
	struct Interval I;
	I.min = (dot - obj->radius);
	I.max = (dot + obj->radius);
	intervalDiv(I, len);

	return I;

    //return Interval( dot - radius, dot + radius ) / len;
}
